
%% GET HS3 DATA
clc; clear all;close all;
load('HS3_COM_2ma.mat')
ts = src1.SampleFrequency;
data = double(src1.Data');
scale = 1e6; % scale for useconds
tps = 1/ts;
n = length(data);
time = ones(1,n);
time = time'.*tps;

% axis data
precision = 6;
x_scale = num2str(tps*scale,precision);

%% Detect periodicity

%data = -data;

do_LP_filter = 0;
if do_LP_filter
    GS_FAC = 29;
    AVG = 9;
    GAUSS = 9;
    data = LP_filter(data,GS_FAC,AVG,GAUSS);
end



fig1 = figure(1);

subplot(2,2,1);
plot(data);xlabel(['Time [',x_scale,' us]']); ylabel('Current [mA]');
title('(a) Cathodic phase ')
grid on;
hold on;
[p,initcross,~,nextcross,midlev] = pulseperiod(data,ts,'MidPct',97);
np = length(p);

%# horizontal line
hy = graph2d.constantline(0, 'Color',[1 0 0]);
changedependvar(hy,'y');

%text(0.5, 1,'\bf Resistive load','HorizontalAlignment' ,'center','VerticalAlignment', 'top')


subplot(2,2,2);
plot(data);xlabel(['Time [',x_scale,' us]']); ylabel('Current [mA]');
title('(b) Anodic phase ')
grid on;
hold on;
[p,initcross,~,nextcross,midlev] = pulseperiod(data,ts,'MidPct',97);
np = length(p);

%# horizontal line
hy = graph2d.constantline(0, 'Color',[1 0 0]);
changedependvar(hy,'y');


subplot(2,2,3);
plot(data);xlabel(['Time [',x_scale,' us]']); ylabel('Current [mA]');
title('(c) IPD ')
grid on;
hold on;
[p,initcross,~,nextcross,midlev] = pulseperiod(data,ts,'MidPct',97);
np = length(p);

%# horizontal line
hy = graph2d.constantline(0, 'Color',[1 0 0]);
changedependvar(hy,'y');

subplot(2,2,4);
plot(data);xlabel(['Time [',x_scale,' us]']); ylabel('Current [mA]');
title('(d) Full period ')
grid on;
hold on;
[p,initcross,~,nextcross,midlev] = pulseperiod(data,ts,'MidPct',97);
np = length(p);

%# horizontal line
hy = graph2d.constantline(0, 'Color',[1 0 0]);
changedependvar(hy,'y');





% find closest index to start and stop points
initcross = initcross.*ts;
nextcross = nextcross.*ts;
startlist = zeros(1,np);
endlist = zeros(1,np);

% fill lists with data indices ## ERROR DUE TO ROUNDING
for i = 1:np
   startlist(i) = round(initcross(i));
   endlist(i) = round(nextcross(i));
end

SAVE_TO_AD = 0;

if SAVE_TO_AD
    data_2ma_4700nF = data(startlist(2):endlist(2));
    save('AD_COMs.mat','data_2ma_4700nF','-append');
end
    
% plot them
plot(initcross,data(startlist),'+g');
plot(nextcross,data(endlist),'or');
hold off;
pp = p*scale;
disp(' ')

%% Integrate signal
if any(startlist)
    % total signal data
    ta = startlist(1);
    tb = endlist(np);
    capt_samples = tb-ta; 
    capt_time = capt_samples*tps*scale; % length of captured signal in microseconds
    data_ti = sum(data(ta:tb));
    data_tm = mean(data(ta:tb));
    tot_cb = data_tm*capt_time;

    fprintf('Signal total integral is %2.5f mA  \n',data_ti);
    fprintf('Signal total mean is %2.5f mA  \n',data_tm);
    fprintf('Signal total CB is %2.5f nC  \n',tot_cb);
    fprintf('Signal total CB per period is %2.5f nC  \n',tot_cb/np);
    disp(' ')

    % pulse data
    data_i = zeros(1,np);
    data_m = zeros(1,np);
    data_cb = zeros(1,np);
    data_peak = zeros(1,np);

    for i = 1:np
        data_i(i) = sum(data(startlist(i):endlist(i)));  % not in use atm
        data_m(i) = mean(data(startlist(i):endlist(i)));
        data_peak(i) = peak2peak(data(startlist(i):endlist(i)));
    end
    data_cb = data_m'.*pp;

    %fprintf('Pulse period is %2.5f microseconds \n', '%2.5f \n',pp );
    %fprintf('Pulse CB is %2.5f nC  \n',data_cb);

    pp_mean = mean(pp);
    pp_var = var(pp);
    cb_mean = mean(data_cb);
    cb_var = var(data_cb);
    pk_mean = mean(data_peak);
    pk_var = var(data_peak);


    fprintf('Pulse peak mean is %2.5f mA \n',pk_mean );
    fprintf('Pulse period mean is %2.5f microseconds \n',pp_mean );
    fprintf('Pulse CB mean is %2.5f nC  \n',cb_mean);
    fprintf('Pulse CB mean deviance from total CB %2.5f percent [BUGGED?] \n',abs(1 - (cb_mean/tot_cb))*100);
    disp(' ')
    fprintf('Pulse peak variance is %2.5f mA \n',pk_var );
    fprintf('Pulse period variance is %2.5f microseconds \n',pp_var );
    fprintf('Pulse CB variance is %2.5f nC  \n',cb_var);
    disp(' ')
    fprintf('Sampling frequency was %2.5f Hz  \n',ts);
    fprintf('Calculations based on %u captured periods  \n',np);


    %% test for normal distribution

    % pulse period
    pp_sig = sqrt(pp_var);
    pp_nd = ztest(pp,pp_mean,pp_sig);

    % Charge balance
    cb_sig = sqrt(cb_var);
    cb_nd = ztest(data_cb,cb_mean,cb_sig);

    if (pp_nd == 0) && (cb_nd == 0)
        disp('Data is assumed normal distributed based on a z test (5 perc sign)')
    end
end


%% FFT analysis

% % extract section for fft
% 
% fft_data = data(4.411e+04:5.406e+04);
% 
% % ffit
% q = pow2(nextpow2(n));  % Transform length
% y = fft(fft_data,q);           % DFT
% power = y.*conj(y)/q;   % Power of the DFT
% 
% % 0-center
% y0 = fftshift(y);          % Rearrange y values
% f0 = (-q/2:q/2-1)*(ts/q);  % 0-centered frequency range
% power0 = y0.*conj(y0)/q;   % 0-centered power
% 
% % try with lp filterd data 
% LP_FAC = 39;
% 
% lp_fft_data = sgolayfilt(fft_data,1,LP_FAC); 
% y_lp = fft(lp_fft_data,q);             
% power_lp = y_lp.*conj(y_lp)/q;  
% y0_lp = fftshift(y_lp);            
% power0_lp = y0_lp.*conj(y0_lp)/q;   

% position figure where you want it and run:
%  borders = get(fig2,'OuterPosition') - get(fig2,'Position')
%  TopBorder = borders(3)+borders(2)
% ah fuckit, just use Position data and this one:
% set(fig2,'units','normalized','outerposition',[ position data here ]);


% 
% fig2 = figure(2);
% set(fig2,'units','normalized','outerposition',[-1.0000   -0.6000    1.0000    (0.9367 + 0.06)]);
% subplot(3,2,1);
% plot(f0,power0)
% xlabel('Frequency (Hz)')
% ylabel('Power')
% title('{\bf 0-Centered Periodogram}')
% 
% subplot(3,2,2);
% phase = unwrap(angle(y0));
% plot(f0,phase*180/pi)
% xlabel('Frequency (Hz)')
% ylabel('Phase (Degrees)')
% title('{\bf Bode Plot}')
% grid on
% 
% subplot(3,2,3);
% plot(fft_data)
% xlabel('Samples')
% ylabel('Amplitude [mA]')
% title('{\bf Raw Signal}')
% subplot(3,2,4);
% plot(lp_fft_data);
% xlabel('Samples')
% ylabel('Amplitude [mA]')
% title('{\bf LP-Filtered Signal}')
% 
% subplot(3,2,5);
% plot(f0,power0_lp)
% xlabel('Frequency (Hz)')
% ylabel('Power')
% title('{\bf 0-Centered LP-Filtered Periodogram}')
% 
% subplot(3,2,6);
% phase = unwrap(angle(y0_lp));
% plot(f0,phase*180/pi)
% xlabel('Frequency (Hz)')
% ylabel('Phase (Degrees)')
% title('{\bf Bode Plot}')
% grid on




